/* Main header file for Takehana */

/* OCTA file header. From src/engine/world.h */
struct octaheader {
  char magic[4];              // "OCTA"
  int version;                // any >8bit quantity is little endian
  int headersize;             // sizeof(header)
  int worldsize;
  int numents;
  int numpvs;
  int lightmaps;
  int blendmap;
  int numvars;
};

enum { ID_VAR, ID_FVAR, ID_SVAR, ID_COMMAND, ID_CCOMMAND, ID_ALIAS };

struct identstack
{
    char *action;
    struct identstack *next;
};

union identval
{
    int i;      // ID_VAR
    float f;    // ID_FVAR
    char *s;    // ID_SVAR
};

union identvalptr
{
    int *i;   // ID_VAR
    float *f; // ID_FVAR
    char **s; // ID_SVAR
};

/* Ident structure, from src/shared/command.h but with C++isms stripped */
struct ident {
  int type;           // one of ID_* above
  char *name;
  union
  {
    int minval;    // ID_VAR
    float minvalf; // ID_FVAR
  };
  union
  {
    int maxval;    // ID_VAR
    float maxvalf; // ID_FVAR
  };
  int override;       // either NO_OVERRIDE, OVERRIDDEN, or value
  union
  {
    void (*fun)(); // ID_VAR, ID_COMMAND, ID_CCOMMAND
    struct identstack *stack;     // ID_ALIAS
  };
  union
  {
    char *narg; // ID_COMMAND, ID_CCOMMAND
    char *action;     // ID_ALIAS
    union identval val;     // ID_VAR, ID_FVAR, ID_SVAR
  };
  union
  {
    void *self;           // ID_COMMAND, ID_CCOMMAND 
    char *isexecuting;    // ID_ALIAS
    union identval overrideval; // ID_VAR, ID_FVAR, ID_SVAR
  };

  union identvalptr storage; // ID_VAR, ID_FVAR, ID_SVAR
  int flags;
};

struct vec
{
    union
    {
        struct { float x, y, z; };
        float v[3];
    };
};

enum { ET_EMPTY=0, ET_LIGHT, ET_MAPMODEL, ET_PLAYERSTART, ET_ENVMAP, ET_PARTICLES, ET_SOUND, ET_SPOTLIGHT, ET_GAMESPECIFIC };

struct entity                                   // persistent map entity
{
    struct vec o;                                      // position
    short attr1, attr2, attr3, attr4, attr5;
    unsigned char type;                       // type is one of the above
    unsigned char reserved;
};

enum { OCTSAV_CHILDREN = 0, OCTSAV_EMPTY, OCTSAV_SOLID, OCTSAV_NORMAL, OCTSAV_LODCUBE };

struct cubeext
{
    unsigned char material;          // empty-space material
    unsigned char visible;           // visible faces of the cube
    unsigned char merged;            // merged faces of the cube
    unsigned char mergeorigin;       // whether this face describes a larger merged face
    /* vtxarray *va;            // vertex array for children, or NULL */
    /* clipplanes *clip;        // collision planes */
    /* surfaceinfo *surfaces;   // lighting info for each surface */
    /* surfacenormals *normals; // per-vertex normals for each surface */
    /* octaentities *ents;      // list of map entites totally inside cube */
    /* mergeinfo *merges;       // bounds of merged surfaces */
    /* int tjoints;             // linked list of t-joints */
};  

struct cube
{
    struct cube *children;          // points to 8 cube structures which are its children, or NULL. -Z first, then -Y, -X
    union
    {
        unsigned char edges[12];     // edges of the cube, each uchar is 2 4bit values denoting the range.
                             // see documentation jpgs for more info.
        unsigned int faces[3];       // 4 edges of each dimension together representing 2 perpendicular faces
    };
    union
    {
        unsigned short texture[6];       // one for each face. same order as orient.
        struct
        {
            unsigned char clipmask, vismask;
            unsigned char vismasks[8];
        };
    };
    struct cubeext *ext;            // extended info for the cube
};

const unsigned int F_EMPTY = 0;             // all edges in the range (0,0)
const unsigned int F_SOLID = 0x80808080;    // all edges in the range (0,8)
