/* txt2oct - Convert textual representation of sauer map to an uncompressed
   binary sauer map */

#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include "takehana.h"

char *octsav_type[] = { "OCTSAV_CHILDREN", "OCTSAV_EMPTY", "OCTSAV_SOLID",
                        "OCTSAV_NORMAL", "OCTSAV_LODCUBE" };

int main() {
  struct octaheader hdr;
  unsigned short size, texnum;
  unsigned char len;
  char gameident[4] = "fps";
  char buf[512];
  char *ptr;
  int i;
  
  /* fill in constant parts of header */
  memcpy(hdr.magic, "OCTA", 4);
  hdr.version = 29;
  hdr.headersize = 36;
  hdr.worldsize = 1024;
  hdr.numents = 0;
  hdr.numpvs = 0;
  hdr.lightmaps = 0;
  hdr.blendmap = 0;
  hdr.numvars = 0;

  /* TODO: get hdr.worldsize and hdr.numvars from the input */

  /* header */
  fwrite(&hdr, sizeof(hdr), 1, stdout);

  /* TODO: variables from the input */

  /* gameident */
  len = 3;
  fwrite(&len, sizeof(unsigned char), 1, stdout);
  fwrite(gameident, sizeof(char), 4, stdout);
  
  size = 0;
  fwrite(&size, sizeof(unsigned short), 1, stdout);/* extraentinfosize */
  fwrite(&size, sizeof(unsigned short), 1, stdout);/* extras.length */
  fwrite(&size, sizeof(unsigned short), 1, stdout);/* texmru.length */

  /* skip out texmru */

  /* skip out entities */

  /* octree data from stdin. format:
     OCTSAV_TYPE TEX1 TEX2 TEX3 TEX4 TEX5 TEX6
     e.g.
     OCTSAV_EMPTY 0 0 0 0 0 0
     OCTSAV_SOLID 2 3 4 5 6 7 */
  while(fgets(buf, 512, stdin)) {
    if(buf[0] == '#') continue;

    for(i = 0; i < 5; i++) {
      if(strncasecmp(buf, octsav_type[i], strlen(octsav_type[i])) == 0) break;
    }

    if(i == 5) {
      fprintf(stderr, "Invalid octsav type: %s\n", buf);
      return 1;
    }

    if(i == OCTSAV_NORMAL) {
      fprintf(stderr, "OCTSAV_NORMAL unsupported.\n");
      return 1;
    }

    /* write the octsav type */
    fputc(i, stdout);

    /* and now the texture numbers unless it's an OCTSAV_CHILDREN */
    if(i != OCTSAV_CHILDREN) {
      ptr = strtok(buf, " \t\n");
      for(i = 0; i < 6; i++) {
        ptr = strtok(NULL, " \t\n");
        if(!ptr) fprintf(stderr, "Warning: Not enough textures for %s.\n",
                         octsav_type[i]);
        texnum = (ptr ? atoi(ptr) : 0);
        fwrite(&texnum, sizeof(unsigned short), 1, stdout);
      }
      
      /* now the mask */
      fputc(0, stdout);
    }
  }
  
  /* now skip lightmaps, pvs, blendmap */

  return 0;
}
